#include "project.h"
#include "stdio.h"
#include "stdlib.h"
#include "tty_utils.h"
#include "i2c_api_utils.h"
#include "i2c_utils.h"

static int      i2c_keep_on  = 1 ;
static uint16_t i2c_loop     = 1 ;
static uint16_t i2c_interval = 100 ; /* 100 ms */

f_list_type i2c_func_list[] = {
    { "help",      do_i2c_help,       "show help message" },
    { "scan",      do_i2c_scan,       "scan I2C bus"      },
    { "slave",     do_i2c_slave,      "set slave address" },
    { "read",      do_i2c_read_byte,  "read 1 byte from the slave" },
    { "write",     do_i2c_write_byte, "write 1 byte to the slave" },
    { "writereg",  do_i2c_write_reg,  "write 1 byte to reg" },
    { "readreg",   do_i2c_read_reg,   "read 1 byte from reg" },
    { "readregs",  do_i2c_read_regs,  "read n bytes starting from the reg" },
    { "status",    do_i2c_status,     "print config status" },
    { "return",    do_i2c_return,     "return to the previous mode" },
    { 0,          0,             0} /* list terminator */
} ;

void do_i2c(void)
{
    func_ptr func ;
    i2c_keep_on = 1 ;
    
    I2C_Start() ;
    
    do_i2c_help() ;
    
    prompt("i2c> ") ;
    
    while(i2c_keep_on) {
        if (get_line()) {
            func = get_func(i2c_func_list, str) ;
            if (func) {
                func() ;
            } else {
                print("Unknown Command: ") ;
                print(str) ;
                print("\n\r") ;
                do_i2c_help() ;
            }
            if (i2c_keep_on) {
                prompt("i2c> ") ;
            }
        }
    }
}


void do_i2c_help(void) 
{
    show_help(i2c_func_list) ;    
}

void do_i2c_return(void) 
{
    i2c_keep_on = 0 ;
}

void do_i2c_loop(void) 
{
    sscanf(str, "%*s %hd", &i2c_loop) ;
}

void do_i2c_interval(void) 
{
    sscanf(str, "%*s %hd", &i2c_interval) ;
}

void do_i2c_measure(void) 
{
    int16_t raw_count, mV ;
    ADC_StartConvert() ;
    ADC_IsEndConversion(ADC_WAIT_FOR_RESULT) ;
    raw_count = ADC_GetResult16(0) ;
    mV = ADC_CountsTo_mVolts(0, raw_count) ;

    if (mV < 0) {
        mV *= -1 ;
        snprintf(str, STR_BUF_LEN, "ADC: -%d.%03d V : ", mV/1000, mV%1000) ;
    } else {
        snprintf(str, STR_BUF_LEN, "ADC:  %d.%03d V : ", mV/1000, mV%1000) ;
    }
    print(str) ;
    snprintf(str, STR_BUF_LEN, "raw count: 0x%04X (%d)\r\n", raw_count&0xFFFF, raw_count) ;
    print(str) ;
}

void do_i2c_status(void) 
{
    snprintf(str, STR_BUF_LEN, "I2C Config: slave address: 0x%02X (%d)\n\r",
        i2c_slave_address, i2c_slave_address) ;
    print(str) ;
}

void do_i2c_scan(void) 
{
    uint8_t address ;
    print("Scanning I2C Addresses... \n\r") ;
    for (address = 0 ; address < 127 ; address++ ) {
        if (i2c_test_address(address)) {
            snprintf(str, STR_BUF_LEN, "0x%02X : device present\r\n", address) ;
            print(str) ;
        }
    }
    print("Scan completed\n\r") ;
}

void do_i2c_slave(void) 
{
    int slave_address ;
    sscanf(str, "%*s %x", &slave_address) ;
    i2c_set_slave_address(slave_address) ;
}

void do_i2c_read_byte(void) 
{
    uint8_t value ;
    value = i2c_readByte(I2C_I2C_ACK_DATA) ;
    snprintf(str, STR_BUF_LEN, "0x%02X\n\r", value) ;
    print(str) ;
}

void do_i2c_write_byte(void) 
{
    uint32_t status ;
    int value ;
    sscanf(str, "%*s %x", &value) ;
    status = i2c_writeByte(value & 0xFF) ;
    if (status == 0) {
        print("OK\n\r") ;
    } else {
        snprintf(str, STR_BUF_LEN, "Error: 0x%X\n\r", status) ;
    }
}

void do_i2c_read_reg(void) 
{
    int reg_address ;
    uint8_t value ;
    
    sscanf(str, "%*s %x", &reg_address) ;
    
    value = i2c_readReg(reg_address) ;
    snprintf(str, STR_BUF_LEN, "0x%02X (%d) \n\r",
        value, value) ;
    print(str) ;
}

void do_i2c_write_reg(void) 
{
    uint32_t status ;
    int reg_address ;
    int value ;
    sscanf(str, "%*s %x %x", &reg_address, &value) ;
    i2c_writeReg(reg_address, value) ;
}

void do_i2c_read_regs(void) 
{
    int reg_address ;
    int num, i, tmp ;
    uint8_t *data ;
    sscanf(str, "%*s %x %d", &reg_address, &num) ;
    data = (uint8_t *)malloc(sizeof(uint8_t) * num) ;
    if (data == 0) {
        print("Not enough memory\r\n") ;
        return ;
    }
    i2c_readRegs(reg_address, data, num) ;
    for (i = 0 ; i < num ; i++) {
        snprintf(str, STR_BUF_LEN, "%02X ", data[i] & 0xFF) ;
        print(str) ;
    }
    print("\n\r") ;
    free( data ) ;
}
